@@ -1,26 +1,33 @@
 /*
  *
- *   base32 encoding/decoding per RFC 4648
+ *  base32 encoding/decoding per RFC 4648
  *
  *   Copyright (C) 2013  Milan Kupcevic
+ *   All rights reserved.
  *
- *   You can redistribute and/or modify this software under the
- *   terms of the GNU General Public License version 3, or any later
- *   version as published by the Free Software Foundation.
+ *  Redistribution and use in source and binary forms, with or without
+ *  modification, are permitted provided that the following conditions are met:
  *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
+ *  1. Redistributions of source code must retain the above copyright notice,
+ *  this list of conditions and the following disclaimer.
  *
- *   You should have received a copy of the GNU General Public License
- *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
- *
- *   GPLv3+
+ *  2. Redistributions in binary form must reproduce the above copyright notice,
+ *  this list of conditions and the following disclaimer in the documentation
+ *  and/or other materials provided with the distribution.
  *
+ *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ *  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ *  POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include "config.h"
 #include "basez.h"
 
 static const unsigned char
@@ -107,72 +114,8 @@ static const unsigned char c32hexd[] =
   0xff, 0xff, 0xff, 0xff,    0xff, 0xff, 0xff, 0xff, /* f8 - ff */
 };
 
-static void
-enc_b32(
-  const unsigned char *bin,
-  const int benc,
-  unsigned char *bout,
-  const int code);
-
-static int
-dec_b32(
-  const unsigned char *bin,
-  unsigned char *bout,
-  const int code);
-
+static
 void
-encode_base32u(
-  const unsigned char *bin,
-  const int benc,
-  unsigned char *bout)
-{
-  enc_b32(bin, benc, bout, 0);
-}
-
-void
-encode_base32l(
-  const unsigned char *bin,
-  const int benc,
-  unsigned char *bout)
-{
-  enc_b32(bin, benc, bout, 1);
-}
-
-int
-decode_base32(
-  const unsigned char *bin,
-  unsigned char *bout)
-{
-  return dec_b32(bin, bout, 0);
-}
-
-void
-encode_base32hexu(
-  const unsigned char *bin,
-  const int benc,
-  unsigned char *bout)
-{
-  enc_b32(bin, benc, bout, 2);
-}
-
-void
-encode_base32hexl(
-  const unsigned char *bin,
-  const int benc,
-  unsigned char *bout)
-{
-  enc_b32(bin, benc, bout, 3);
-}
-
-int
-decode_base32hex(
-  const unsigned char *bin,
-  unsigned char *bout)
-{
-  return dec_b32(bin, bout, 2);
-}
-
-static void
 enc_b32(
   const unsigned char *bin,
   const int benc,
@@ -244,7 +187,8 @@ enc_b32(
   }
 }
 
-static int
+static
+int
 dec_b32(
   const unsigned char *bin,
   unsigned char *bout,
@@ -348,3 +292,55 @@ dec_b32(
   }
   return 5;
 }
+
+void
+encode_base32u(
+  const unsigned char *bin,
+  const int benc,
+  unsigned char *bout)
+{
+  enc_b32(bin, benc, bout, 0);
+}
+
+void
+encode_base32l(
+  const unsigned char *bin,
+  const int benc,
+  unsigned char *bout)
+{
+  enc_b32(bin, benc, bout, 1);
+}
+
+int
+decode_base32(
+  const unsigned char *bin,
+  unsigned char *bout)
+{
+  return dec_b32(bin, bout, 0);
+}
+
+void
+encode_base32hexu(
+  const unsigned char *bin,
+  const int benc,
+  unsigned char *bout)
+{
+  enc_b32(bin, benc, bout, 2);
+}
+
+void
+encode_base32hexl(
+  const unsigned char *bin,
+  const int benc,
+  unsigned char *bout)
+{
+  enc_b32(bin, benc, bout, 3);
+}
+
+int
+decode_base32hex(
+  const unsigned char *bin,
+  unsigned char *bout)
+{
+  return dec_b32(bin, bout, 2);
+}
