@@ -56,25 +56,26 @@ typedef enum opt_crlf {off, on} opt_crlf;
 typedef enum opt_case {upper, lower, any} opt_case;
 typedef enum code {plain, hex, url, mime, pem} code;
 
-const char* base_name(const char *command_path_name);
-void version();
-void help();
-void encode_b64(const code variant);
-void decode_b64(const code variant);
-void encode_b32(const code variant);
-void decode_b32(const code variant);
-void encode_b16();
-void decode_b16();
-void invalid_input();
-void internal_error();
-void set_wrap(const char *arg);
-void sys_err(const char *msg);
+static const char* base_name(const char *command_path_name);
+static void version();
+static void help();
+static void encode_b64(const code variant);
+static void decode_b64(const code variant);
+static void encode_b32(const code variant);
+static void decode_b32(const code variant);
+static void encode_b16();
+static void decode_b16();
+static void invalid_input();
+static void internal_error();
+static void set_wrap(const char *arg);
+static void sys_err(const char *msg);
 
 static const char *command_name;
 static opt_crlf o_crlf = off;
 static opt_ignore o_i = nl;
 static opt_case o_c = any;
 static FILE *file_in;
+static FILE *file_out;
 static int wrap = -1;
 
 int
@@ -84,6 +85,7 @@ main (
 {
   int i;
   const char *filename_in = NULL;
+  const char *filename_out = NULL;
   opt_base o_b = b64;
   opt_mode o_m = encode;
   command_name = NULL;
@@ -95,14 +97,16 @@ main (
     command_name = "basez";
 
   file_in = stdin;
+  file_out = stdout;
 
   if (strcmp(command_name, "base16") == 0)     o_b = b16;
+  if (strcmp(command_name, "base16plain") == 0)o_b = b16;
   if (strcmp(command_name, "hex") == 0)        o_b = b16;
   if (strcmp(command_name, "unhex") == 0)    { o_b = b16; o_m = decode; }
   if (strcmp(command_name, "base32") == 0)     o_b = b32;
   if (strcmp(command_name, "base32plain") == 0)o_b = b32;
   if (strcmp(command_name, "base32hex") == 0)  o_b = b32hex;
-  if (strcmp(command_name, "base64") == 0)   { o_b = b64; wrap = 76; }
+  if (strcmp(command_name, "base64") == 0)   { o_b = b64; }
   if (strcmp(command_name, "base64plain") == 0)o_b = b64;
   if (strcmp(command_name, "base64url") == 0)  o_b = b64url;
   if (strcmp(command_name, "base64mime") == 0){o_b = b64mime;
@@ -114,10 +118,7 @@ main (
 
   for (i = 1; i < argc; i++)
   {
-    if (strcmp(argv[i], "-") == 0)           { continue; }
     if (strcmp(argv[i], "--") == 0)          { continue; }
-    if((strcmp(argv[i], "-i") == 0)
-    || (strcmp(argv[i], "--input") == 0))    { continue; }
     if (strcmp(argv[i], "--version") == 0)   { version(); }
     if((strcmp(argv[i], "-h") == 0)
     || (strcmp(argv[i], "--help") == 0))     { help(); }
@@ -130,21 +131,32 @@ main (
     || (strcmp(argv[i], "--ignore-garbage") == 0))
                                              { o_i = all; continue; }
     if((strcmp(argv[i], "-x") == 0)
-    || (strcmp(argv[i], "--b16") == 0))      { o_b = b16; continue; }
+    || (strcmp(argv[i], "--b16") == 0)
+    || (strcmp(argv[i], "--hex") == 0)
+    || (strcmp(argv[i], "--base16plain") == 0)
+    || (strcmp(argv[i], "--base16") == 0))   { o_b = b16; continue; }
     if((strcmp(argv[i], "-j") == 0)
-    || (strcmp(argv[i], "--b32") == 0))      { o_b = b32; continue; }
+    || (strcmp(argv[i], "--b32") == 0)
+    || (strcmp(argv[i], "--base32plain") == 0)
+    || (strcmp(argv[i], "--base32") == 0))   { o_b = b32; continue; }
     if((strcmp(argv[i], "-e") == 0)
-    || (strcmp(argv[i], "--b32hex") == 0))   { o_b = b32hex; continue; }
+    || (strcmp(argv[i], "--b32hex") == 0)
+    || (strcmp(argv[i], "--base32hex") == 0)){ o_b = b32hex; continue; }
     if((strcmp(argv[i], "-a") == 0)
-    || (strcmp(argv[i], "--b64") == 0))      { o_b = b64; continue; }
+    || (strcmp(argv[i], "--b64") == 0)
+    || (strcmp(argv[i], "--base64plain") == 0)
+    || (strcmp(argv[i], "--base64") == 0))   { o_b = b64; continue; }
     if((strcmp(argv[i], "-u") == 0)
-    || (strcmp(argv[i], "--b64url") == 0))   { o_b = b64url; continue; }
+    || (strcmp(argv[i], "--b64url") == 0)
+    || (strcmp(argv[i], "--base64url") == 0)){ o_b = b64url; continue; }
     if((strcmp(argv[i], "-m") == 0)
-    || (strcmp(argv[i], "--b64mime") == 0))  { o_b = b64mime;
+    || (strcmp(argv[i], "--b64mime") == 0)
+    || (strcmp(argv[i], "--base64mime")== 0)){ o_b = b64mime;
                                               wrap = 76;
                                                o_i = all; continue; }
     if((strcmp(argv[i], "-p") == 0)
-    || (strcmp(argv[i], "--b64pem") == 0))   { o_b = b64pem;
+    || (strcmp(argv[i], "--b64pem") == 0)
+    || (strcmp(argv[i], "--base64pem") == 0)){ o_b = b64pem;
                                               wrap = 64;
                                                o_i = space; continue; }
     if((strcmp(argv[i], "-t") == 0)
@@ -159,6 +171,59 @@ main (
     if((strcmp(argv[i], "-l") == 0)
     || (strcmp(argv[i], "--lower-case") == 0))
                                              { o_c = lower; continue; }
+
+    if((strcmp(argv[i], "-o") == 0)
+    || (strcmp(argv[i], "--output") == 0))
+    {
+      if (i + 1 < argc)
+      {
+        i++;
+        filename_out = argv[i];
+        continue;
+      }
+      else
+      {
+        fprintf(stderr, "%s: %s: invalid argument\n", command_name, argv[i]);
+        exit(EXIT_FAILURE);
+      }
+    }
+    if ((strncmp(argv[i], "--output=", 9) == 0))
+    {
+      filename_out = argv[i]+9;
+      continue;
+    }
+    if ((strncmp(argv[i], "-o", 2) == 0))
+    {
+      filename_out = argv[i]+2;
+      continue;
+    }
+
+    if((strcmp(argv[i], "-i") == 0)
+    || (strcmp(argv[i], "--input") == 0))
+    {
+      if (i + 1 < argc)
+      {
+        i++;
+        filename_in = argv[i];
+        continue;
+      }
+      else
+      {
+        fprintf(stderr, "%s: %s: invalid argument\n", command_name, argv[i]);
+        exit(EXIT_FAILURE);
+      }
+    }
+    if ((strncmp(argv[i], "--input=", 8) == 0))
+    {
+      filename_in = argv[i]+8;
+      continue;
+    }
+    if ((strncmp(argv[i], "-i", 2) == 0))
+    {
+      filename_in = argv[i]+2;
+      continue;
+    }
+
     if((strcmp(argv[i], "-w") == 0)
     || (strcmp(argv[i], "--wrap") == 0)
     || (strcmp(argv[i], "-b") == 0)
@@ -193,7 +258,7 @@ main (
       continue;
     }
 
-    if (*argv[i] == '-')
+    if (*argv[i] == '-' && *(argv[i]+1) != '\0')
     { 
       fprintf(stderr, "%s: %s: invalid option\n", command_name, argv[i]);
       exit(EXIT_FAILURE);
@@ -221,9 +286,12 @@ main (
       o_crlf = off;
   }
 
+  if (filename_in != NULL && *filename_in == '-' && *(filename_in+1) == '\0')
+    filename_in = NULL;
+
   if (filename_in != NULL)
   {
-    file_in = fopen(filename_in, o_m == encode && o_crlf == off ? "rb" : "rt");
+    file_in = fopen(filename_in, o_m == encode && o_crlf == off ? "rb":"r");
     if (file_in == NULL)
       sys_err(filename_in);
   }
@@ -234,10 +302,24 @@ main (
       if (_setmode(_fileno(stdin), _O_BINARY) == -1)
         sys_err("stdin");
   }
+#endif /* _WIN32 */
 
-  if (o_m == decode && o_crlf == off)
-    if (_setmode(_fileno(stdout), _O_BINARY) == -1)
-      sys_err("stdout");
+  if (filename_out != NULL && *filename_out == '-' && *(filename_out+1) == '\0')
+    filename_out = NULL;
+
+  if (filename_out != NULL)
+  {
+    file_out = fopen(filename_out, o_m == decode && o_crlf == off ? "wb":"w");
+    if (file_out == NULL)
+      sys_err(filename_out);
+  }
+#if defined(_WIN32)
+  else
+  {
+    if (o_m == decode && o_crlf == off)
+      if (_setmode(_fileno(stdout), _O_BINARY) == -1)
+        sys_err("stdout");
+  }
 #endif /* _WIN32 */
 
   switch (o_m)
@@ -305,6 +387,7 @@ main (
   return EXIT_SUCCESS;
 }
 
+static
 void
 set_wrap (
   const char *arg)
@@ -316,11 +399,12 @@ set_wrap (
   }
   else
   {
-    fprintf(stderr,"%s: %s: invalid wrap value\n", command_name, arg);
+    fprintf(stderr,"%s: %s: invalid value\n", command_name, arg);
     exit(EXIT_FAILURE);
   }
 }
 
+static
 const char*
 base_name (
   const char *command_path_name)
@@ -343,6 +427,7 @@ base_name (
   return ret;
 }
 
+static
 void
 version ()
 {
@@ -371,6 +456,7 @@ APPNAME ") " VERSION "\n"
   exit(EXIT_SUCCESS);
 }
 
+static
 void
 help ()
 {
@@ -378,7 +464,9 @@ help ()
 "\n"
 "Usage: basez  [OPTION]... [FILE]\n"
 "   or: hex [OPTION]... [FILE]\n"
+#if UNHEX
 "   or: unhex [OPTION]... [FILE]\n"
+#endif
 "   or: base16 [OPTION]... [FILE]\n"
 #if B32
 "   or: base32 [OPTION]... [FILE]\n"
@@ -448,6 +536,10 @@ APPNAME
   puts(
 "\n"
 "Options:\n"
+"When no FILE is specified or when FILE is -, read standard input. \n"
+"When multiple conflicting options appear, the last option wins.\n"
+  );
+  puts(
 "   -d, -D, --decode\n"
 "                    Decode. \n"
 "                    By default ignore the space and newline characters. \n"
@@ -461,22 +553,22 @@ APPNAME
   puts(
 "   -s, --ignore-all-space\n"
 "                    Ignore all white and non-printable ASCII characters \n"
-"                    on decoding. Default for base64pem command.\n"
+"                    on decoding. Default for base64pem decoding.\n"
 "\n"
 "   -g, --ignore-garbage\n"
 "                    Ignore all characters outside of the encode \n"
 "                    character-set on decoding. Default for \n"
-"                    base64mime command.\n"
+"                    base64mime decoding.\n"
 "\n"
-"   -x, --b16        Base16 coding. Default for base16, hex and \n"
+"   -x, --hex, --base16\n"
+"                    Base16 coding. Default for base16, hex and \n"
 "                    unhex commands.\n"
   );
   puts(
-"\n"
-"   -j, --b32        Base32 coding. Default for base32 and base32plain \n"
+"   -j, --base32     Base32 coding. Default for base32 and base32plain \n"
 "                    commands.\n"
 "\n"
-"   -e  --b32hex     Base32hex coding. Default for base32hex command.\n"
+"   -e  --base32hex  Base32hex coding. Default for base32hex command.\n"
 "\n"
 "   -c, --capitals, --upper-case\n"
 "                    Output upper case letters on encoding, if applicable.\n"
@@ -484,52 +576,54 @@ APPNAME
 "   -l, --lower-case\n"
 "                    Output lower case letters on encoding, if applicable.\n"
 "\n"
-"   -a, --b64        Base64 coding. Default for basez, base64 and \n"
-"                    base64plain commands."
+"   -a, --base64     Base64 coding. Default for basez, base64 and \n"
+"                    base64plain commands.\n"
   );
   puts(
-"   -u, --b64url     Base64url coding. Default for base64url command.\n"
+"   -u, --base64url  Base64url coding. Default for base64url command.\n"
 "\n"
-"   -p, --b64pem     PEM printable coding. Default for base64pem command.\n"
-"\n"
-"   -m, --b64mime    MIME base64 coding. Default for base64mime command.\n"
+"   -p, --base64pem  PEM printable coding. Default for base64pem command.\n"
 "\n"
+"   -m, --base64mime\n"
+"                    MIME base64 coding. Default for base64mime command.\n"
+  );
+  puts(
 "   -t, --text       Convert native text line breaks into CRLF sequences \n"
 "                    prior to encoding or convert CRLF sequences into \n"
 "                    native text line breaks after decoding. This option \n"
 "                    applies to all variants of base64 codings.\n"
   );
   puts(
-"\n"
 "   -w N, -b N, --wrap=N, --break=N\n"
 "                    Set encoded stream line length limit to N characters. \n"
-"                    The default value is 76 for base64 and base64mime \n"
-"                    commands, 64 for base64pem command, infinity for \n"
-"                    all other commands.\n"
+"                    The default value is 76 for base64mime encode, 64 for\n"
+"                    base64pem encode, infinity for all other encodes.\n"
 "                    To disable any appearance of EOL characters in the \n"
 "                    encoded stream, set to 0.\n"
   );
   puts(
+"   -i FILEIN, --input=FILEIN\n"
+"                    Input file. Default is standard input.\n"
+"                    When FILEIN is -, read stdin.\n"
 "\n"
-"   -i, --input      Ignore this option for compatibility reasons.\n"
+"   -o FILEOUT, --output=FILEOUT\n"
+"                    Output file. Default is standard output.\n"
+"                    When FILEOUT is -, write to stdout.\n"
 "\n"
 "   -h, --help       Display help.\n"
 "\n"
 "       --version    Display program version information.\n"
-"\n"
-"When no FILE is specified or when FILE is -, read standard input. \n"
-"When multiple conflicting options appear, the last option wins.\n"
-"\n"
   );
   puts(
+"\n"
 "Examples:\n"
 "Base16 decode a string:\n"
 "\n"
-"    echo 4a6f650a | unhex\n"
+"    echo 4a6f650a | hex --decode\n"
 "\n"
 "Inspect output of a command:\n"
 "\n"
-"    echo Joe | hex -w2\n"
+"    echo Joe | hex --wrap 2\n"
 "\n"
 "Base32 encode a string:\n"
 "\n"
@@ -546,9 +640,9 @@ APPNAME
 "Base64 encode a file per RFC 4648\n"
 "\n"
 "    base64plain FILE > base64.rfc4648.file\n"
-"\n"
   );
   puts(
+"\n"
 "Report bugs to <bug-" EMAIL "> with a 'bug:' somewhere in the subject line.\n"
 "\n"
 "These commands are parts of the "
@@ -568,12 +662,63 @@ APPNAME "]        <http://" WEB ">\n"
   exit(EXIT_SUCCESS);
 }
 
+#define BUFF 512
+
+static unsigned char bof[BUFF];
+static size_t bofwcount = 0;
+static unsigned char bif[BUFF];
+static int bifincount = 0;
+static int bifrcount = 0;
+
+static
+void
+flushout()
+{
+  if ((fwrite(&bof[0], 1, bofwcount, file_out)) != bofwcount)
+    sys_err("");
+  bofwcount = 0;
+}
+
+static
+unsigned char
+writechar(const unsigned char in)
+{
+  bof[bofwcount++] = in;
+  if (bofwcount == BUFF)
+    flushout();
+  return in;
+}
+
+static
+void
+readin()
+{
+  if ((bifincount = fread(&bif[0], 1, BUFF, file_in)) != BUFF)
+    if (ferror(file_in))
+      sys_err("");
+  bifrcount = 0;
+}
+
+static
+int
+readchar()
+{
+  if (bifincount == bifrcount)
+  {
+    readin();
+    if (bifincount == 0)
+      return EOF;
+  }
+  return bif[bifrcount++];
+}
+
+static
 void
 encode_b64(
   const code variant)
 {
   int cin;
-  int cout = -1;
+  unsigned char cout = '\0';
   unsigned char buffin[3];
   unsigned char remainder = '\0';
   unsigned char buffout[4];
@@ -589,7 +734,7 @@ encode_b64(
       buffin[buffin_count++] = remainder;
       remainder = '\0';
     }
-    while (buffin_count < 3 && ((cin = getc(file_in)) != EOF))
+    while (buffin_count < 3 && ((cin = readchar()) != EOF))
     {
       buffin[buffin_count] = cin;
       if (o_crlf == on && buffin[buffin_count] == '\n')
@@ -607,9 +752,6 @@ encode_b64(
       buffin_count++;
     }
 
-    if (ferror(file_in))
-      sys_err("");
-
     if (buffin_count > 0)
     {
       switch (variant)
@@ -627,8 +769,7 @@ encode_b64(
       buffout_count = 0;
       for ( ; buffout_count < 4; buffout_count++)
       {
-        if ((cout = putc(buffout[buffout_count], stdout)) == EOF)
-          sys_err("");
+        cout = writechar(buffout[buffout_count]);
         switch (wrap)
         {
           case -1:
@@ -637,8 +778,7 @@ encode_b64(
           default:
             if (++cout_count >= wrap)
             {
-              if ((cout = putc('\n', stdout)) == EOF)
-                sys_err("");
+              cout = writechar('\n');
               cout_count=0;
             }
         }
@@ -647,12 +787,14 @@ encode_b64(
   } while (buffin_count == 3);
 
   if ((cout != '\n') && (wrap != 0))
-  {
-    if (putc('\n', stdout) == EOF)
-      sys_err("");
-  }
+    writechar('\n');
+
+  flushout();
+  if (fclose(file_out) != 0)
+    sys_err("");
 }
 
+static
 void
 decode_b64(
   const code variant)
@@ -669,7 +811,7 @@ decode_b64(
   do
   {
     buffin_count = 0;
-    while (buffin_count < 4 && ((cin = getc(file_in)) != EOF))
+    while (buffin_count < 4 && ((cin = readchar()) != EOF))
     {
       switch (o_i)
       {
@@ -709,9 +851,6 @@ decode_b64(
       buffin[buffin_count++] = cin;
     }
 
-    if (ferror(file_in))
-      sys_err("");
-
     switch (buffin_count)
     {
       case 4:
@@ -741,8 +880,7 @@ decode_b64(
             {
               if (o_crlf == on && buffout[buffout_count] == '\r')
                 continue;
-              if (putc(buffout[buffout_count], stdout) == EOF)
-                sys_err("");
+              writechar(buffout[buffout_count]);
             }
             continue;
             break;
@@ -764,14 +902,19 @@ decode_b64(
     }
 
   } while (run == 1);
+
+  flushout();
+  if (fclose(file_out) != 0)
+    sys_err("");
 }
 
+static
 void
 encode_b32(
   const code variant)
 {
   int cin;
-  int cout = -1;
+  unsigned char cout = '\0';
   unsigned char buffin[5];
   unsigned char buffout[8];
   int buffin_count;
@@ -781,14 +924,11 @@ encode_b32(
   do
   {
     buffin_count = 0;
-    while (buffin_count < 5 && ((cin = getc(file_in)) != EOF))
+    while (buffin_count < 5 && ((cin = readchar()) != EOF))
     {
       buffin[buffin_count++] = cin;
     }
 
-    if (ferror(file_in))
-      sys_err("");
-
     if (buffin_count > 0)
     {
       switch (variant)
@@ -820,8 +960,7 @@ encode_b32(
       buffout_count = 0;
       for ( ; buffout_count < 8; buffout_count++)
       {
-        if ((cout = putc(buffout[buffout_count], stdout)) == EOF)
-          sys_err("");
+        cout = writechar(buffout[buffout_count]);
         switch (wrap)
         {
           case -1:
@@ -830,8 +969,7 @@ encode_b32(
           default:
             if (++cout_count >= wrap)
             {
-              if ((cout = putc('\n', stdout)) == EOF)
-                sys_err("");
+              cout = writechar('\n');
               cout_count=0;
             }
         }
@@ -840,12 +978,14 @@ encode_b32(
   } while (buffin_count == 5);
 
   if ((cout != '\n') && (wrap != 0))
-  {
-    if (putc('\n', stdout) == EOF)
-      sys_err("");
-  }
+    writechar('\n');
+
+  flushout();
+  if (fclose(file_out) != 0)
+    sys_err("");
 }
 
+static
 void
 decode_b32(
   const code variant)
@@ -862,7 +1002,7 @@ decode_b32(
   do
   {
     buffin_count = 0;
-    while (buffin_count < 8 && ((cin = getc(file_in)) != EOF))
+    while (buffin_count < 8 && ((cin = readchar()) != EOF))
     {
       switch (o_i)
       {
@@ -900,9 +1040,6 @@ decode_b32(
       buffin[buffin_count++] = cin;
     }
 
-    if (ferror(file_in))
-      sys_err("");
-
     switch (buffin_count)
     {
       case 8:
@@ -929,10 +1066,7 @@ decode_b32(
           case 5:
             buffout_count = 0;
             for ( ; buffout_count < decoded_count; buffout_count++)
-            {
-              if (putc(buffout[buffout_count], stdout) == EOF)
-                sys_err("");
-            }
+              writechar(buffout[buffout_count]);
             continue;
             break;
           case 0:
@@ -957,19 +1091,24 @@ decode_b32(
     }
 
   } while (run == 1);
+
+  flushout();
+  if (fclose(file_out) != 0)
+    sys_err("");
 }
 
+static
 void 
 encode_b16 ()
 {
   int cin;
-  int cout = -1;
+  unsigned char cout = '\0';
   unsigned char buffin;
   unsigned char buffout[2];
   int buffout_count;
   int cout_count = 0;
 
-  while ((cin = getc(file_in)) != EOF)
+  while ((cin = readchar()) != EOF)
   {
     buffin = cin;
     switch (o_c)
@@ -984,8 +1123,7 @@ encode_b16 ()
     buffout_count = 0;
     for ( ; buffout_count < 2; buffout_count++)
     {
-      if ((cout = putc(buffout[buffout_count], stdout)) == EOF)
-        sys_err("");
+      cout = writechar(buffout[buffout_count]);
       switch (wrap)
       {
         case -1:
@@ -994,23 +1132,22 @@ encode_b16 ()
         default:
           if (++cout_count >= wrap)
           {
-            if ((cout = putc('\n', stdout)) == EOF)
-              sys_err("");
+            cout = writechar('\n');
             cout_count=0;
           }
       }
     }
   }
-  if (ferror(file_in))
-    sys_err("");
 
   if ((cout != '\n') && (wrap != 0))
-  {
-    if (putc('\n', stdout) == EOF)
-      sys_err("");
-  }
+    writechar('\n');
+
+  flushout();
+  if (fclose(file_out) != 0)
+    sys_err("");
 }
 
+static
 void 
 decode_b16()
 {
@@ -1023,7 +1160,7 @@ decode_b16()
   do
   {
     buffin_count = 0;
-    while (buffin_count < 2 && ((cin = getc(file_in)) != EOF))
+    while (buffin_count < 2 && ((cin = readchar()) != EOF))
     {
       switch (o_i)
       {
@@ -1047,16 +1184,12 @@ decode_b16()
       buffin[buffin_count++] = cin;
     }
 
-    if (ferror(file_in))
-      sys_err("");
-
     switch (buffin_count)
     {
       case 2:
         if (decode_base16(&buffin[0], &buffout) == 1)
         {
-          if (putc(buffout, stdout) == EOF)
-            sys_err("");
+          writechar(buffout);
           continue;
         }
       case 1:
@@ -1069,8 +1202,12 @@ decode_b16()
 
   } while (run == 1);
 
+  flushout();
+  if (fclose(file_out) != 0)
+    sys_err("");
 }
 
+static
 void
 internal_error ()
 {
@@ -1078,6 +1215,7 @@ internal_error ()
   exit(EXIT_FAILURE);
 }
 
+static
 void
 invalid_input ()
 {
@@ -1085,6 +1223,7 @@ invalid_input ()
   exit(EXIT_FAILURE);
 }
 
+static
 void
 sys_err (
   const char *msg)
